<?php
/*
Plugin Name: Advanced Auto SKU Assigner
Description: Automatically assigns an SKU to products and variations upon creation. Supports custom SKU formats, bulk updates, logging, and automatic SKU generation during imports.
Version: 1.2
Author: EasyToolsWP
*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Hovedplugin-klassen
class Advanced_Auto_SKU_Assigner {
    
    public function __construct() {
        // Hooks for SKU-tildeling ved produkt- og variantoprettelse
        add_action('save_post_product', array($this, 'assign_sku_to_product'));
        add_action('save_post_product_variation', array($this, 'assign_sku_to_variation'));

        // Tilføj indstillingsside
        add_action('admin_menu', array($this, 'add_plugin_page'));
        add_action('admin_init', array($this, 'page_init'));

        // Logføring ved SKU-tildeling
        add_action('updated_post_meta', array($this, 'log_sku_assignment'), 10, 4);

        // Bulk update handling
        add_action('admin_post_bulk_update_skus', array($this, 'bulk_update_skus'));
    }

    public function run() {
        // Eventuelle initieringer
    }

    public function assign_sku_to_product($post_id) {
        $product = wc_get_product($post_id);
        if (!$product->get_sku()) {
            $sku = $this->generate_sku($product);
            $product->set_sku($sku);
            $product->save();
        }
    }

    public function assign_sku_to_variation($post_id) {
        $variation = wc_get_product($post_id);
        if (!$variation->get_sku()) {
            $sku = $this->generate_sku($variation);
            $variation->set_sku($sku);
            $variation->save();
        }
    }

    private function generate_sku($product) {
        $options = get_option('auto_sku_assigner_options');
        $length = isset($options['sku_length']) ? $options['sku_length'] : 6;
        $prefix = isset($options['sku_prefix']) ? $options['sku_prefix'] : '';
        $suffix = isset($options['sku_suffix']) ? $options['sku_suffix'] : '';

        $sku_body = wp_generate_password($length, false, false);

        return $prefix . $sku_body . $suffix;
    }

    public function add_plugin_page() {
        add_options_page(
            'Advanced Auto SKU Assigner', 
            'Advanced Auto SKU Assigner', 
            'manage_options', 
            'advanced-auto-sku-assigner-admin', 
            array($this, 'create_admin_page')
        );
    }

    public function create_admin_page() {
        ?>
        <div class="wrap">
            <h1>Advanced Auto SKU Assigner</h1>
            <form method="post" action="options.php">
                <?php
                settings_fields('auto_sku_assigner_option_group');
                do_settings_sections('advanced-auto-sku-assigner-admin');
                submit_button();
                ?>
            </form>
            <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
                <input type="hidden" name="action" value="bulk_update_skus">
                <?php submit_button('Bulk Update SKUs'); ?>
            </form>
        </div>
        <?php
    }

    public function page_init() {
        register_setting(
            'auto_sku_assigner_option_group',
            'auto_sku_assigner_options',
            array($this, 'sanitize')
        );

        add_settings_section(
            'setting_section_id',
            'SKU Settings',
            array($this, 'print_section_info'),
            'advanced-auto-sku-assigner-admin'
        );

        add_settings_field(
            'sku_length',
            'Number of characters in SKU',
            array($this, 'sku_length_callback'),
            'advanced-auto-sku-assigner-admin',
            'setting_section_id'
        );

        add_settings_field(
            'sku_prefix',
            'Prefix SKU',
            array($this, 'sku_prefix_callback'),
            'advanced-auto-sku-assigner-admin',
            'setting_section_id'
        );

        add_settings_field(
            'sku_suffix',
            'Suffix SKU',
            array($this, 'sku_suffix_callback'),
            'advanced-auto-sku-assigner-admin',
            'setting_section_id'
        );

        add_settings_field(
            'log_sku_assignments',
            'Log SKU Assignments',
            array($this, 'log_sku_assignments_callback'),
            'advanced-auto-sku-assigner-admin',
            'setting_section_id'
        );
    }

    public function sanitize($input) {
        $sanitized_input = array();
        if (isset($input['sku_length']))
            $sanitized_input['sku_length'] = absint($input['sku_length']);
        if (isset($input['sku_prefix']))
            $sanitized_input['sku_prefix'] = sanitize_text_field($input['sku_prefix']);
        if (isset($input['sku_suffix']))
            $sanitized_input['sku_suffix'] = sanitize_text_field($input['sku_suffix']);
        if (isset($input['log_sku_assignments']))
            $sanitized_input['log_sku_assignments'] = (bool) $input['log_sku_assignments'];
        return $sanitized_input;
    }

    public function print_section_info() {
        print 'Enter your settings below:';
    }

    public function sku_length_callback() {
        $options = get_option('auto_sku_assigner_options');
        printf(
            '<input type="text" id="sku_length" name="auto_sku_assigner_options[sku_length]" value="%s" />',
            isset($options['sku_length']) ? esc_attr($options['sku_length']) : ''
        );
    }

    public function sku_prefix_callback() {
        $options = get_option('auto_sku_assigner_options');
        printf(
            '<input type="text" id="sku_prefix" name="auto_sku_assigner_options[sku_prefix]" value="%s" />',
            isset($options['sku_prefix']) ? esc_attr($options['sku_prefix']) : ''
        );
    }

    public function sku_suffix_callback() {
        $options = get_option('auto_sku_assigner_options');
        printf(
            '<input type="text" id="sku_suffix" name="auto_sku_assigner_options[sku_suffix]" value="%s" />',
            isset($options['sku_suffix']) ? esc_attr($options['sku_suffix']) : ''
        );
    }

    public function log_sku_assignments_callback() {
        $options = get_option('auto_sku_assigner_options');
        printf(
            '<input type="checkbox" id="log_sku_assignments" name="auto_sku_assigner_options[log_sku_assignments]" %s />',
            isset($options['log_sku_assignments']) && $options['log_sku_assignments'] ? 'checked' : ''
        );
    }

    public function bulk_update_skus() {
        $args = array(
            'post_type' => array('product', 'product_variation'),
            'posts_per_page' => -1,
            'post_status' => array('publish', 'draft', 'pending')
        );
        $query = new WP_Query($args);
        foreach ($query->posts as $post) {
            $product = wc_get_product($post->ID);
            if (!$product->get_sku()) {
                $sku = $this->generate_sku($product);
                $product->set_sku($sku);
                $product->save();
            }
        }
        wp_redirect(add_query_arg('bulk_update_status', 'success', admin_url('options-general.php?page=advanced-auto-sku-assigner-admin')));
        exit;
    }
}

// Initialiser pluginet
if (is_admin()) {
    $advanced_auto_sku_assigner = new Advanced_Auto_SKU_Assigner();
}
?>
